@@ -17,12 +17,42 @@
 
 #include "xexpression.hpp"
 #include "xiterable.hpp"
+#include "xsemantic.hpp"
 #include "xstrides.hpp"
 #include "xutils.hpp"
 
 namespace xt
 {
 
+    /*************************
+     * xindex_view extension *
+     *************************/
+
+    namespace extension
+    {
+        template <class Tag, class CT, class I>
+        struct xindex_view_base_impl;
+
+        template <class CT, class I>
+        struct xindex_view_base_impl<xtensor_expression_tag, CT, I>
+        {
+            using type = xtensor_empty_base;
+        };
+
+        template <class CT, class I>
+        struct xindex_view_base
+            : xindex_view_base_impl<xexpression_tag_t<CT>, CT, I>
+        {
+        };
+
+        template <class CT, class I>
+        using xindex_view_base_t = typename xindex_view_base<CT, I>::type;
+    }
+
+    /***************
+     * xindex_view *
+     ***************/
+
     template <class CT, class I>
     class xindex_view;
 
@@ -41,10 +71,6 @@ namespace xt
         using stepper = xindexed_stepper<xindex_view<CT, I>, false>;
     };
 
-    /***************
-     * xindex_view *
-     ***************/
-
     /**
      * @class xindex_view
      * @brief View of an xexpression from vector of indices.
@@ -61,13 +87,17 @@ namespace xt
      */
     template <class CT, class I>
     class xindex_view : public xview_semantic<xindex_view<CT, I>>,
-                        public xiterable<xindex_view<CT, I>>
+                        public xiterable<xindex_view<CT, I>>,
+                        public extension::xindex_view_base_t<CT, I>
     {
     public:
 
         using self_type = xindex_view<CT, I>;
         using xexpression_type = std::decay_t<CT>;
         using semantic_base = xview_semantic<self_type>;
+        
+        using extension_base = extension::xindex_view_base_t<CT, I>;
+        using expression_tag = typename extension_base::expression_tag;
 
         using value_type = typename xexpression_type::value_type;
         using reference = typename xexpression_type::reference;
@@ -105,7 +135,7 @@ namespace xt
         size_type dimension() const noexcept;
         const inner_shape_type& shape() const noexcept;
         layout_type layout() const noexcept;
-        
+
         template <class T>
         void fill(const T& value);
 
@@ -135,11 +165,14 @@ namespace xt
         template <class It>
         const_reference element(It first, It last) const;
 
+        xexpression_type& expression() noexcept;
+        const xexpression_type& expression() const noexcept;
+
         template <class O>
         bool broadcast_shape(O& shape, bool reuse_cache = false) const;
 
         template <class O>
-        bool is_trivial_broadcast(const O& /*strides*/) const noexcept;
+        bool has_linear_assign(const O& /*strides*/) const noexcept;
 
         template <class ST>
         stepper stepper_begin(const ST& shape);
@@ -151,6 +184,12 @@ namespace xt
         template <class ST>
         const_stepper stepper_end(const ST& shape, layout_type) const;
 
+        template <class E>
+        using rebind_t = xindex_view<E, I>;
+
+        template <class E>
+        rebind_t<E> build_index_view(E&& e) const;
+
     private:
 
         CT m_e;
@@ -213,7 +252,7 @@ namespace xt
 
         template <class E>
         disable_xexpression<E, self_type&> operator%=(const E&);
-        
+
     private:
 
         template <class F>
@@ -234,7 +273,7 @@ namespace xt
     /**
      * Constructs an xindex_view, selecting the indices specified by \a indices.
      * The resulting xexpression has a 1D shape with a length of n for n indices.
-     * 
+     *
      * @param e the underlying xexpression for this view
      * @param indices the indices to select
      */
@@ -318,7 +357,7 @@ namespace xt
      * @name Data
      */
     //@{
-    
+
     /**
      * Fills the view with the given value.
      * @param value the value to fill the view with.
@@ -465,6 +504,24 @@ namespace xt
     {
         return m_e[m_indices[(*first)]];
     }
+
+    /**
+     * Returns a reference to the underlying expression of the view.
+     */
+    template <class CT, class I>
+    inline auto xindex_view<CT, I>::expression() noexcept -> xexpression_type&
+    {
+        return m_e;
+    }
+
+    /**
+     * Returns a constant reference to the underlying expression of the view.
+     */
+    template <class CT, class I>
+    inline auto xindex_view<CT, I>::expression() const noexcept -> const xexpression_type&
+    {
+        return m_e;
+    }
     //@}
 
     /**
@@ -485,13 +542,13 @@ namespace xt
     }
 
     /**
-     * Compares the specified strides with those of the container to see whether
-     * the broadcasting is trivial.
-     * @return a boolean indicating whether the broadcasting is trivial
+     * Checks whether the xindex_view can be linearly assigned to an expression
+     * with the specified strides.
+     * @return a boolean indicating whether a linear assign is possible
      */
     template <class CT, class I>
     template <class O>
-    inline bool xindex_view<CT, I>::is_trivial_broadcast(const O& /*strides*/) const noexcept
+    inline bool xindex_view<CT, I>::has_linear_assign(const O& /*strides*/) const noexcept
     {
         return false;
     }
@@ -533,6 +590,13 @@ namespace xt
         return const_stepper(this, offset, true);
     }
 
+    template <class CT, class I>
+    template <class E>
+    inline auto xindex_view<CT, I>::build_index_view(E&& e) const -> rebind_t<E>
+    {
+        return rebind_t<E>(std::forward<E>(e), indices_type(m_indices));
+    }
+
     /******************************
      * xfiltration implementation *
      ******************************/
@@ -543,7 +607,7 @@ namespace xt
     //@{
     /**
      * Constructs a xfiltration on the given expression \c e, selecting
-     * the elements matching the specified \c condition. 
+     * the elements matching the specified \c condition.
      *
      * @param e the \ref xexpression to filter.
      * @param condition the filtering \ref xexpression to apply.
@@ -647,12 +711,12 @@ namespace xt
 
     /**
      * @brief creates an indexview from a container of indices.
-     *        
+     *
      * Returns a 1D view with the elements at \a indices selected.
      *
      * @param e the underlying xexpression
      * @param indices the indices to select
-     * 
+     *
      * \code{.cpp}
      * xarray<double> a = {{1,5,3}, {4,5,6}};
      * b = index_view(a, {{0, 0}, {1, 0}, {1, 1}});
@@ -690,13 +754,14 @@ namespace xt
 
     /**
      * @brief creates a view into \a e filtered by \a condition.
-     *        
+     *
      * Returns a 1D view with the elements selected where \a condition evaluates to \em true.
-     * This is equivalent to \verbatim{index_view(e, where(condition));}\endverbatim
+     * This is equivalent to \verbatim{index_view(e, argwhere(condition));}\endverbatim
      * The returned view is not optimal if you just want to assign a scalar to the filtered
      * elements. In that case, you should consider using the \ref filtration function
      * instead.
      *
+     * @tparam L the traversal order
      * @param e the underlying xexpression
      * @param condition xexpression with shape of \a e which selects indices
      *
@@ -708,10 +773,10 @@ namespace xt
      *
      * \sa filtration
      */
-    template <class E, class O>
+    template <layout_type L = XTENSOR_DEFAULT_TRAVERSAL, class E, class O>
     inline auto filter(E&& e, O&& condition) noexcept
     {
-        auto indices = where(std::forward<O>(condition));
+        auto indices = argwhere<L>(std::forward<O>(condition));
         using view_type = xindex_view<xclosure_t<E>, decltype(indices)>;
         return view_type(std::forward<E>(e), std::move(indices));
     }
